#include <fnmatch.h>

#include <glib.h>

#include "egg-file-filter.h"

struct _EggFileFilter
{
	gboolean is_custom;
	gchar *name;
	union
	{
		struct
		{
			EggCustomFilterFunc filter_fn;
			GFreeFunc free_fn;
			gpointer data;
		} custom;
		struct
		{
			GList *extns;
		} file;
	} d;
};

EggFileFilter *egg_file_filter_new(const gchar *name, const gchar **extns)
{
	EggFileFilter *filter;
	const gchar **extn;

	g_return_val_if_fail(name && extns, NULL);
	filter = g_new0(EggFileFilter, 1);
	filter->is_custom = FALSE;
	filter->name = g_strdup(name);
	for (extn = extns; *extn; ++ extn)
		filter->d.file.extns = g_list_prepend(filter->d.file.extns, g_strdup(*extn));
	return filter;
}

EggFileFilter *egg_file_filter_new_custom(const gchar *name, EggCustomFilterFunc filter_fn
  , GFreeFunc free_fn, gpointer data)
{
	EggFileFilter *filter;

	g_return_val_if_fail(name && filter_fn, NULL);
	filter = g_new0(EggFileFilter, 1);
	filter->is_custom = TRUE;
	filter->name = g_strdup(name);
	filter->d.custom.filter_fn = filter_fn;
	filter->d.custom.free_fn = free_fn;
	filter->d.custom.data = data;
	return filter;
}

void egg_file_filter_free(EggFileFilter *filter)
{
	g_return_if_fail(filter);
	g_free(filter->name);
	if (filter->is_custom)
	{
		if (filter->d.custom.free_fn && filter->d.custom.data)
			filter->d.custom.free_fn(filter->d.custom.data);
	}
	else
	{
		GList *tmp;
		for (tmp = filter->d.file.extns; tmp; tmp = g_list_next(tmp))
			g_free(tmp->data);
		g_list_free(filter->d.file.extns);
	}
	g_free(filter);
}

gboolean egg_file_filter_add_extn(EggFileFilter *filter, const gchar *extn)
{
	g_return_val_if_fail(filter && extn && !filter->is_custom, FALSE);
	filter->d.file.extns = g_list_prepend(filter->d.file.extns, g_strdup(extn));
	return TRUE;
}

static gboolean match_file_by_extn(const gchar *file, GList *extns)
{
	GList *tmp;
	for (tmp = extns; tmp; tmp = g_list_next(tmp))
	{
		if (0 == fnmatch((const gchar *) tmp->data, file, FNM_FILE_NAME))
			return TRUE;
	}
	return FALSE;
}

gboolean egg_file_filter_match(EggFileFilter *filter, const gchar *file)
{
	g_return_val_if_fail(filter && file, FALSE);
	if (filter->is_custom)
		return filter->d.custom.filter_fn(file, filter->d.custom.data);
	else
		return match_file_by_extn(file, filter->d.file.extns);
}
