/*
** Filtering file names using various criteria
** Licence: GNU LGPL
** Author: Biswapesh Chattopadhyay <biswapesh_chatterjee@tcscal.co.in>
*/

#ifndef EGG_FILE_FILTER_H
#define EGG_FILE_FILTER_H

/*! \file
** File filters are used by the file selection widget to provide file name filtering
** in order to determine which files should be shown and which should be hidden. One
** inbuilt filter, 'Hidden Files' is provided by default with the file selection widget
** through the FSView interface. This class provides another simple extension based filter.
** Roughly modelled after the Swing FileFilter class. The interface is not GObject based
** since there are no object requirements and it would add unnecessary bloat.
** References:
**   1) http://java.sun.com/j2se/1.4/docs/api/javax/swing/filechooser/FileFilter.html
*/

#ifdef __cplusplus
extern "C"
{
#endif

/*! EggFileFilter is a opaque structure to ensure binary compatibility */
typedef struct _EggFileFilter EggFileFilter;

/*!
** Extension based filter takes a name and a NULL terminated list of patterns.
** Example:
** gchar *extns[] = { "*.png", "*.gif", "*.jpg", NULL };
** egg_file_filter_new("Image Files", extns)
*/
EggFileFilter *egg_file_filter_new(const gchar *name, const gchar **extns);

/*!
** Signature of function required by custom filters. The funciton should take a path/uri
** and the pointer passed to egg_file_filter_new_custom() as parameters.
** \return TRUE if the filter succeded or FALSE if the filter failed.
*/
typedef gboolean (*EggCustomFilterFunc) (const gchar *path, gpointer data);

/*! Custom filters take a EggCustomFilterFunc function as a parameter.
** \param name The text that is shown in the filter combo
** \param func The filter function
** \param free_fn This function will be called with the 'data' pointer when the structure is being
** free()d.
** \param data Any data that the function might need. This is passed as-is to func().
** Example:
**   gboolean mime_filter(const gchar *path, gpointer data);
**   myfilter = egg_file_filter_new_custom("Image Files", mime_filter, "image/");
*/
EggFileFilter *egg_file_filter_new_custom(const gchar *name, EggCustomFilterFunc filter_fn
  , GFreeFunc free_fn, gpointer data);

/*! Destroy a EggFileFilter structure */
void egg_file_filter_free(EggFileFilter *filter);

/*!
** Add a pattern to an existing extension filter.
** Ex: egg_file_filter_add_extn(filter, "*.tif")
*/
gboolean egg_file_filter_add_extn(EggFileFilter *filter, const gchar *extn);

/*! Check if a file matches the given filter */
gboolean egg_file_filter_match(EggFileFilter *filter, const gchar *file);

#ifdef __cplusplus
}
#endif

#endif /* EGG_FILE_FILTER_H */
