/*
** This is the new GTK extensible fileselection widget. See README for details.
** Licence: GNU LGPL
** Author: Biswapesh Chattopadhyay <biswapesh_chatterjee@tcscal.co.in>
*/

#ifndef EGG_FILESEL_H
#define EGG_FILESEL_H

#ifdef __cplusplus
extern "C"
{
#endif


#define EGG_TYPE_FILESEL                 (egg_filesel_get_type ())
#define EGG_FILESEL(obj)                 (GTK_CHECK_CAST ((obj), EGG_TYPE_FILESEL EggFileSel))
#define EGG_FILESEL_CLASS(klass)         (GTK_CHECK_CLASS_CAST ((klass), EGG_TYPE_FILESEL, EggFileSelClass))
#define EGG_IS_FILE_SELECTION(obj)       (GTK_CHECK_TYPE ((obj), EGG_TYPE_FILESEL))
#define EGG_IS_FILESEL_CLASS(klass)      (GTK_CHECK_CLASS_TYPE ((klass), EGG_TYPE_FILESEL))
#define EGG_FILESEL_GET_CLASS(obj)       (GTK_CHECK_GET_CLASS ((obj), EGG_TYPE_FILESEL, EggFileSelClass))

typedef struct _EggFileSel EggFileSel;
typedef struct _EggFileSelClass  EggFileSelClass;
typedef struct _EggFileSelPrivate EggFileSelPrivate;

/*! The Gtk file selector is a GtkDialog. Individual members are private and can only be accessed
** through the API. See egg_filesel_get_widget() for details.
*/
struct _EggFileSel
{
	GtkDialog parent_instance; /*!< The parent class object (GtkDialog) */
	EggFileSelPrivate *private_data; /*!< Private implementation specific data */
};

/*! The Fileselection class structure */
struct _EggFileSelClass
{
  GtkDialogClass parent_class; /*!< The parent dialog class */

  /* Padding for future expansion */
  void (*_gtk_reserved1) (void);
  void (*_gtk_reserved2) (void);
  void (*_gtk_reserved3) (void);
  void (*_gtk_reserved4) (void);
};

/*! Get the object type */
GtkType egg_filesel_get_type (void) G_GNUC_CONST;

/*! File selection type based on the purpose of the widget. This will automatically
** set some sensible default values for various parameters. Of course, they can
** always be overridden using other functions.
*/
typedef enum _EggFileSelType
{
  EGG_FILESEL_TYPE_OPEN, /*!< Open file dialog - OK button: _("Open") */
  EGG_FILESEL_TYPE_SAVE, /*!< Save file dialog - OK button: _("Save") */
  EGG_FILESEL_TYPE_OTHER /*!< Other type (e.g. Attach) - OK button: _("OK") */
} EggFileSelType;

/* Functions for creating a new file selection dialog */

/*! The normal way to create a file selection widget. Pass NULL for 'title' to get a default
** title depending on type. Pass NULL for 'start' to start from the current directory.
** Flags will be set to reasonable defaults based on type.
*/
GtkWidget *egg_filesel_new(EggFileSelType type, const gchar *title, const gchar *start);

/*! Same as egg_filesel_new, except that a file system view object needs to be passed for
** a different FS backend */
GtkWidget *egg_filesel_new_with_fsview(EggFileSelType type, const EggFSView *fsv
  , const gchar *title, const gchar *start);

/* Functions to get/set selected files */

/*! Get the current selected filenames. This returns a NULL terminated list of strings. You
** must free() the string list using g_strfreev() or equivalent.
*/
gchar **egg_filesel_get_selection(GtkWidget *filesel);

/*! Set the current selected filenames.
** \param files A NULL terminated list of filenames. If the filename does not exist, it is
** silently ignored. You can also specify patterns here since matching is done using fnmatch().
** \return Number of files successfully selected.
** Example:
** char *patterns[] = { "*.png", "*.gif", NULL };
** egg_filesel_set_selection(filesel, patterns);
*/
gboolean egg_filesel_set_selection(GtkWidget *filesel, const gchar **files);

/*!
** Flags to fine-tune the behavior of the filesel dialog. You can set multiple properties
** by OR-ing them bitwise.
** Decision: perhaps it would be better to segregate these into different properties ?
** e.g:
**  1) Selection Type: single/multiple, files/directories/both, existing/non-existing
**  2) Show: files/directories/both, hidden, preview, list/details, filter
**  3) Others: warn if exists/noexists, etc.
*/
typedef enum _EggFileSelFlag
{
  EGG_FILESEL_SELECT_MULTIPLE = 1 << 1, /*!< Allow multiple selections */
  EGG_FILESEL_SELECT_EXISTING = 1 << 2, /*!< Allow selection(s) of existing files/dirs only */
  EGG_FILESEL_SELECT_FILES = 1 << 3, /*!< Allow selection(s) of files only */
  EGG_FILESEL_SELECT_DIRS = 1 << 4, /*!< Allow selection(s) of directories only */
  EGG_FILESEL_SHOW_FILES = 1 << 5, /*!< Show regular files */
  EGG_FILESEL_SHOW_DIR = 1 << 6, /*!< Show directories */
  EGG_FILESEL_SHOW_HIDDEN = 1 << 7, /*!< Show hidden files and directories */
  EGG_FILESEL_SHOW_PREVIEW = 1 << 8, /*!< Show file preview if possible */
  EGG_FILESEL_SHOW_DETAILS = 1 << 9, /*!< Show file/dir details (normal is a simple list) */
  EGG_FILESEL_SHOW_FILTER = 1 << 10, /*!< Show selection filter */
  EGG_FILESEL_WARN_EXIST = 1 << 11, /*!< Warn if selecting an existing file/directory */
  EGG_FILESEL_WARN_NOEXIST = 1 << 12, /*!< Warn if selecting a non-existing file/directory */
} EggFileSelFlag;

/*! Setting and getting fileselection flags. The 'flag' variable should be bitwise OR of
** EggFileSelFlag types.
** For example, show hidden files with:
** egg_filesel_set_flags(filesel, egg_filesel_get_flags(filesel) | EGG_FILESEL_SHOW_HIDDEN)
*/


/*! Get the current property flags for the selector */
int egg_filesel_get_flags(GtkWidget *filesel);

/*! Set the property flags for the selector */
void egg_filesel_set_flags(GtkWidget *filesel, int flags);

/*! Sets the starting directory to the given value. NULL will set it to the HOME directory */
void egg_filesel_set_start(GtkWidget *filesel, const gchar *start);

/*!
** You can get the individual widgets used by the file selector. These widgets are guaranteed to be
** part of the file selector implementation. You can also add your widgets to the file selector at
** different positions (left, right, top, bottom). The following enum defines the component widgets
** available and also the places where you can add your own widgets. Note that you can get the widget
** at any position but you can only *set* the custom widgets, not the inbuilt ones.
*/
typedef enum _EggFileSelWidgetType
{
	EGG_FILESEL_WIDGET_OK, /*!< The OK button */
	EGG_FILESEL_WIDGET_CANCEL, /*!< The Cancel button */
	EGG_FILESEL_WIDGET_HELP, /*!< The Help button */
	EGG_FILESEL_WIDGET_ENTRY, /*!< Entry box containing selected file name */
	EGG_FILESEL_WIDGET_FILTER, /*!< The filter combo box */
	EGG_FILESEL_WIDGET_LEFT, /*!< Custom widget to the left of the selector */
	EGG_FILESEL_WIDGET_RIGHT, /*!< Custom widget to the right of the selector */
	EGG_FILESEL_WIDGET_TOP, /*!< Custom widget at the top of the selector */
	EGG_FILESEL_WIDGET_BOTTON, /*! Custom widget at the bottom of the selector */
} EggFileSelWidgetType;

/*! Get the widget at the given position */
GtkWidget *egg_filesel_widget_get(GtkWidget *filesel, EggFileSelWidgetType type);

/*! Set the widget at the given position (only works for the custom widgets */
gboolean egg_filesel_widget_set(GtkWidget *filesel, EggFileSelWidgetType type, GtkWidget *widget);

#ifdef __cplusplus
}
#endif

#endif /* EGG_FILESEL_H */
