#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/stat.h>

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "egg-fsview.h"

struct _EggFSView
{
	EggFSVNew fn_new;
	EggFSVGetHome fn_home;
	EggFSVGetDefault fn_default;
	EggFSVGetRoots fn_roots;
	EggFSVGetChildren fn_children;
	EggFSVGetParent fn_parent;
	EggFSVGetDetails fn_details;
	EggFSVGetData fn_data;
	EggFSVCreate fn_create;
	EggFSVRename fn_rename;
	EggFSVFree fn_free;
	gpointer extra;
};

static gpointer new_file(const gchar *uri, gpointer extra)
{
	const gchar *ptr = uri;

	g_return_val_if_fail(uri, NULL);
	if (0 == strncmp(uri, "file:", 5))
		ptr += 5;
	return g_strdup(ptr);
}

static gpointer get_home(gpointer extra)
{
	return g_strdup(getenv("HOME"));
}

static gpointer get_default(gpointer extra)
{
	char cwd[PATH_MAX];

	if (NULL == getcwd(cwd, PATH_MAX))
		return NULL;
	else
		return g_strdup(cwd);
}

static GList *get_roots(gpointer extra)
{
	return g_list_prepend(NULL, g_strdup("/"));
}

static GList *get_children(gpointer parent, gpointer extra)
{
	GList *children = NULL;
	DIR *dir;
	struct dirent *entry;

	g_return_val_if_fail(parent, NULL);
	if (NULL == (dir = opendir((const gchar *) parent)))
	{
		perror((const gchar *) parent);
		return NULL;
	}
	while (NULL != (entry = readdir(dir)))
	{
		if ((0 != strcmp(entry->d_name, ".")) && (0 != strcmp(entry->d_name, "..")))
		{
			children = g_list_prepend(children, g_strdup(entry->d_name));
		}
	}
	closedir(dir);
	return children;
}

static gpointer get_parent(gpointer file, gpointer extra)
{
	gchar *ptr;

	g_return_val_if_fail(file, NULL);
	ptr = strrchr(file, '/');
	if (NULL == ptr)
		return g_strdup(file);
	else
		return g_strndup((const gchar *) file, ((gulong) ptr - (gulong) file));
}

#define DET(x) (EGG_FILEINFO_ ## x == (details & EGG_FILEINFO_ ## x))
static gboolean get_file_details(gpointer file, int details, EggFileInfo *info, gpointer extra)
{
	g_return_val_if_fail(file && details && info, FALSE);

	if DET(NAME)
	{
		if (info->name)
			g_free(info->name);
		info->name = strrchr((char *) file, '/');
		if (info->name)
			info->name = g_strdup(info->name + 1);
		else
			info->name = g_strdup((char *) file);
	}
	if DET(PATH)
	{
		if (info->path)
			g_free(info->path);
		info->path = g_strdup((char *) file);
	}
	if DET(URI)
	{
		if (info->uri)
			g_free(info->uri);
		info->uri = g_strconcat("file:", (char *) file, NULL);
	}
	if DET(ISHIDDEN)
	{
		char *name = strrchr((char *) file, '/');
		if (name)
			++ name;
		else
			name = (char *) file;
		if ('.' == name[0])
			info->is_hidden = TRUE;
		else
			info->is_hidden = FALSE;
	}
	if DET(ISROOT)
	{
		if (0 == strcmp((char *) file, "/"))
			info->is_root = TRUE;
		else
			info->is_root = FALSE;
	}
	if (DET(EXISTS) || DET(SIZE) || DET(ISDIR) || DET(MTIME))
	{
		struct stat s;

		if (0 == stat((char *) file, &s))
			info->exists = TRUE;
		else
			info->exists = FALSE;
		if DET(SIZE)
		{
			if (info->exists)
				info->size = s.st_size;
			else
				info->size = 0;
		}
		if DET(MTIME)
		{
			if (info->exists)
				info->mtime = s.st_mtime;
		}
		if DET(ISDIR)
		{
			if (info->exists && S_ISDIR(s.st_mode))
				info->is_dir = TRUE;
			else
				info->is_dir = FALSE;
		}
	}
	return TRUE;
}

static gboolean get_file_data(gpointer file, gpointer extra, gint *len, gpointer *data)
{
	int fd;
	struct stat s;

	g_return_val_if_fail(file && len && data, FALSE);
	return TRUE;
}

#define DIR_CREATE_MODE S_IRWXU

static gboolean create_file(gpointer dir, const gchar *name, gboolean is_dir, gpointer extra)
{
	char path[PATH_MAX];

	g_return_val_if_fail(dir && name, FALSE);
	g_snprintf(path, PATH_MAX, "%s/%s", (const gchar *) dir, name);
	if (is_dir)
	{
		if (0 != mkdir(path, DIR_CREATE_MODE))
		{
			perror(path);
			return FALSE;
		}
	}
	else
	{
		if (0 > open(path, O_CREAT | O_EXCL))
		{
			perror(path);
			return FALSE;
		}
	}
	return TRUE;
}

static gboolean rename_file(gpointer dir, const gchar *old, const gchar *new, gpointer extra)
{
	char old_file[PATH_MAX], new_file[PATH_MAX];
	struct stat s;

	g_return_val_if_fail(dir && old && new, FALSE);
	g_snprintf(old_file, PATH_MAX, "%s/%s", (const gchar *) dir, old);
	g_snprintf(new_file, PATH_MAX, "%s/%s", (const gchar *) dir, new);
	if (0 == stat(new_file, &s))
	{
		g_warning("%s: File exists", new_file);
		return FALSE;
	}
	if (0 != rename(old_file, new_file))
	{
		g_warning("%s -> %s: %s", old_file, new_file, strerror(errno));
		return FALSE;
	}
	return TRUE;
}

static void free_file(gpointer file, gpointer extra)
{
	g_return_if_fail(file);
	g_free(file);
}

static EggFSView local_fsview = {
	new_file,
	get_home,
	get_default,
	get_roots,
	get_children,
	get_parent,
	get_file_details,
	get_file_data,
	create_file,
	rename_file,
	free_file,
	NULL
};

EggFSView *egg_fsview_new(gpointer extra)
{
	EggFSView *fsv = g_new0(EggFSView, 1);
	memcpy(fsv, &local_fsview, sizeof(local_fsview));
	fsv->extra = extra;
	return fsv;
}
