/*
** GTK File and File System Views
** Licence: GNU LGPL
** Author: Biswapesh Chattopadhyay <biswapesh_chatterjee@tcscal.co.in>
*/

#ifndef EGG_FSVIEW_H
#define EGG_FSVIEW_H

/*! \file
** Different backends can be supported by the file selector using the EggFSView
** structure. This structure maintains the functions required to perform various
** file operations required by the fileselector. A NULL value in any function
** indicates that the function is not available for this backend. The fileselector
** will come only with the 'Local filesystem' backend. Other backends should be
** provided by the components themselves. For example, the GnomeVFS module should
** provide a backend for using GnomeVFS. A backend is built using egg_fsview_new()
** followed by a series of calls to egg_fsview_register_function().
** The FSView class is loosely based on the Java FileView and FileSystemView classes.
** References:
**   1) http://java.sun.com/j2se/1.4/docs/api/javax/swing/filechooser/FileView.html
**   2) http://java.sun.com/j2se/1.4/docs/api/javax/swing/filechooser/FileSystemView.html
*/

#ifdef __cplusplus
extern "C"
{
#endif

/*! The details required for a file. Should be able to be used in bitwise OR-ed manner.
** Required by FSView in order to know what details are required for the file. Also see
** EggFileInfo structure. To preserve binary compatibility, you should only append to
** this list with values in powers of 2.
*/
typedef enum _EggFileInfoType
{
  EGG_FILEINFO_NAME = 1 << 1, /*!< Display name of the file */
  EGG_FILEINFO_PATH = 1 << 2, /*!< Full path of the file (returned by the file selector) */
  EGG_FILEINFO_URI = 1 << 3, /*!< Full URI of the file (defaults to file://path if NULL) */
  EGG_FILEINFO_OWNER = 1 << 4, /*!< Owner of the file */
  EGG_FILEINFO_GROUP = 1 << 5, /*!< Group of the file */
  EGG_FILEINFO_PERM = 1 << 6, /*!, Permissions in UNIX standard (owner, group, other) */
  EGG_FILEINFO_ICON = 1 << 7, /*!< System icon associated with the file */
  EGG_FILEINFO_SIZE = 1 << 8, /*!< Size of the file in bytes */
  EGG_FILEINFO_EXISTS = 1 << 9, /*!< Whether this file exists */
  EGG_FILEINFO_ISHIDDEN = 1 << 10, /*!< Is the file hidden */
  EGG_FILEINFO_ISDIR = 1 << 11, /*!< Is the file traversible (i.e. is this a directory) */
  EGG_FILEINFO_ISROOT = 1 << 12, /*!< Is this a root directory (i.e, cannot go up) */
  EGG_FILEINFO_MTIME = 1 << 13 /*!< Modification time of the file */
} EggFileInfoType;

/*! Structure containing file details which should be filled by the get_details function.
** This structure has to be public so that other filesystem view implementations can fill in
** this structure directly. Note that this structure will be free()d automatically, along
** with the char * elements - so always strdup() data before populating it. The elements
** have one to one correspondence with the EggFileInfoType enum. The 'unused' members
** are there for future expansion.
*/
typedef struct _EggFileInfo
{
  gchar *name; /*!< Name of the file */
  gchar *path; /*!< Full path of the file */
  gchar *uri; /*!< URI of the file */
  gchar *owner; /*!< Owner name */
  gchar *group; /*!< Group name */
  gulong perm; /*!< Permissions in UNIX standard */
  gpointer icon; /*!< File icon (XPM data ?) */
  gulong size; /*!< File size */
  gboolean exists; /*!< Whether a file by this name/URI exists */
  gboolean is_hidden; /*!< If the file is a hidden file */
  gboolean is_dir; /*!< If the file is a directory */
  gboolean is_root; /*!< If this is a root directory */
  time_t mtime; /*!< Modification time of the file */
  gpointer data; /*!< File data pointer */
  gpointer unused1; /*!< Added for future expansion without losing binary compatibility */
  gpointer unused2; /*!< Added for future expansion without losing binary compatibility */
  gpointer unused3; /*!< Added for future expansion without losing binary compatibility */
  gpointer unused4; /*!< Added for future expansion without losing binary compatibility */
} EggFileInfo;

/*! List of functions that can be set in a EggFSView. They all have different signatures.
** To preserve binary compatibility, extra functions should only be appended.
** The signatures of the various functions are given below
*/

/*! Create an opaque pointer containing file data from the given URI. This pointer will be
** passed to various other functions such as GetDetails to get the information required
** by the file selector.
** \param uri URI from which the structure is to be created.
** \param extra The argument passed to egg_fsview_new()
** \return A not-NULL pointer on success, NULL on failure.
*/
typedef gpointer (*EggFSVNew) (const gchar *uri, gpointer extra);

/*! Gets the home directory */
typedef gpointer (*EggFSVGetHome) (gpointer extra);

/*! Gets the default directory - might not be same as the home directory on some systems */
typedef gpointer (*EggFSVGetDefault) (gpointer extra);

/*! Gets a list of root directories (might be more than one for non-UNIX systems */
typedef GList* (*EggFSVGetRoots) (gpointer extra);

/*! Gets a list of children for the given directory */
typedef GList *(*EggFSVGetChildren) (gpointer parent, gpointer extra);

/*! Gets the parent directory for the given directory/file */
typedef gpointer (*EggFSVGetParent) (gpointer file, gpointer extra);

/*! Populate the EggFileInfo structure with the required details.
\param file The file whose details are required
\param details Bitwise OR of EggFileInfoType values specifying required details
\param info Pointer to the EggFileInfo structure that is to be populated.
\param extra Data passed to egg_fsview_new()
\return TRUE on success, FALSE on failure
*/
typedef gboolean (*EggFSVGetDetails) (gpointer file, int details, EggFileInfo *info, gpointer extra);

/*! Get file data for the given file pointer.
** \param file Pointer to the file data
** \param extra Data passed to egg_fsview_new()
** \param len Pointer to the amount of data requested. Should be set to actual length of data.
** \param data Pointer to a void pointer which should contain the fetched data.
** \return TRUE on success, FALSE on failure
*/
typedef gboolean (*EggFSVGetData) (gpointer file, gpointer extra, gint *len, gpointer *data);

/*! Create a new file or directory.
** \param dir The current directory
** \param name The name of the entity to create
** \param is_dir Whether to create a directory
** \param extra Data passed to egg_fsview_new()
** \return TRUE on success, FALSE on failure
*/
typedef gboolean (*EggFSVCreate) (gpointer dir, const gchar *name, gboolean is_dir, gpointer extra);

/*! Rename a file or directory.
\param dir The current directory
\param old The old name of the file/directory
\param new The new name of the file/directory
\param extra The argument passed to egg_fsview_new()
\return TRUE on success, FALSE on failure
*/
typedef gboolean (*EggFSVRename) (gpointer dir, const gchar *old, const gchar *new, gpointer extra);

/*! Free resources associated with the file data pointer
** \param file The file data
** \param extra The argument passed to egg_fsview_new()
*/
typedef void (*EggFSVFree) (gpointer file, gpointer extra);

/*!
** Enumerated list of methods required to be provided by a FSView implementation. To
** maintain binary compatibility, please only append to this list.
*/
typedef enum _EggFSViewMethodType
{
  EGG_FSV_NEW, /*!< Create a FileSytem specific structure from a URI */
  EGG_FSV_GETHOME, /*!< Get the HOME directory */
  EGG_FSV_GETDEFAULT, /*!< Get the default directory (on first invocation) */
  EGG_FSV_GETROOTS, /*!< Get all root directories ('/' for UNIX, 'C:', 'D:' for Windows */
  EGG_FSV_GETCHILDREN, /*!< Get list of children */
  EGG_FSV_GETPARENT, /*!< Get parent of this file */
  EGG_FSV_GETDETAILS, /*!< Populate the required information for the file */
  EGG_FSV_GETDATA, /*!< Get file data (required for fiel previews, for example) */
  EGG_FSV_CREATE, /*!< Creates a new file/directory */
  EGG_FSV_RENAME, /*!< Renames a file/directory */
  EGG_FSV_FREE /*!< Free()s memory allocated by file data */
} EggFSViewMethodType;

/*! EggFileView is an opaque structure containing information about the file system */
typedef struct _EggFSView EggFSView;

/*!
** Creates a new FSView structure. By default, the functions get populated with the functions
** for local filesystem browsing. You can override these functions with your own (as long
** as they match the signatures given above.
** \param extra This can contain any information that might need to be passed to the
** file system functions.
*/
EggFSView *egg_fsview_new(gpointer extra);

/*!
** Create a copy of a file system view. Useful when an existing view serves 90% of your
** purpose but you just want to override a few functions (say, geticon). Note that the
** 'extra' bit gets shallow-copied, so if you're using tricks with extra, you have to be
** careful that the parent FSView does not get free()d.
*/
EggFSView *egg_fsview_copy(const EggFSView *fsv);

/*!
** Set the function for the given method type. Note that though the signature here is that
** of a void function, the actual signature of the function must match the type as given
** above. You should cast the function to a GVoidFunc and pass it as the func argument.
** \param fsv The FSView strture pointer
** \param type The method type you want to override. See EggFSViewMethodType.
** \param func The function corresponding to the method. NULL indicates that this facility
** is not available for this file system view
** \return TRUE if registration succeded, FALSE otherwise.
*/
gboolean egg_fsview_set_method(EggFSView *fsv, EggFSViewMethodType type, GVoidFunc func);

/*!
** This function is the file selector's interface to the file system view. It will return
** the function corresponding to the given type. Note that NULL is a valid return value,
** indicating that this particular method is not supported by this backend. For example,
** a GnomeVFS protocol may not support creation of new files/directories, in which case
** egg_fsview_get_method(fsv, EGG_FSV_CREATE) will return NULL.
*/
GVoidFunc egg_fsview_get_method(EggFSView *fsv, EggFSViewMethodType type);

/*!
** Destroys resources allocated by egg_fsview_new(). You should pass the function which
** should be used to free() the 'extra' parameter passed to egg_fsview_new()
*/
void egg_fsview_free(EggFSView *fsv, GFreeFunc free_fn);

/*! The default file system view provided. This should be sufficient for most uses */
const EggFSView *egg_local_fsview(void);

#ifdef __cplusplus
}
#endif

#endif /* EGG_FSVIEW_H */
